--
-- Scythe
--
-- Author: Stijn Wopereis
-- Description: scythe hand tool
-- Name: Scythe
-- Hide: yes
--
-- Copyright (c) Creative Mesh, 2023

---@class Scythe
Scythe = {}
Scythe.INTERACTION_DELAY = 2000 --ms

local Scythe_mt = Class(Scythe, HandTool)
InitObjectClass(Scythe, "Scythe")

g_xmlManager:addInitSchemaFunction(function()
    local schema = HandTool.xmlSchema
    schema:setXMLSpecializationType("Scythe")
    Scythe.registerXMLPaths(schema, "handTool")
    AnimatedHandTool.registerXMLPaths(schema, "handTool")
    schema:setXMLSpecializationType()
end)

function Scythe.registerXMLPaths(schema, basePath)
    SoundManager.registerSampleXMLPaths(schema, basePath .. ".scythe.sounds", "cut")
    EffectManager.registerEffectXMLPaths(schema, basePath .. ".scythe.effects")

    schema:register(XMLValueType.STRING, basePath .. ".scythe.workingAnimation#name", "Working animation name")
    schema:register(XMLValueType.FLOAT, basePath .. ".scythe.workingAnimation#workingDirection", "Working animation direction if is working", 1)
    schema:register(XMLValueType.FLOAT, basePath .. ".scythe.workingAnimation#releasingDirection", "Working animation direction if is releasing", "-workingDirection")
end

---@return Scythe
function Scythe.new(isServer, isClient, customMt)
    local self = Scythe:superClass().new(isServer, isClient, customMt or Scythe_mt)

    self.animations = AnimatedHandTool.new(isServer, isClient)
    self.playerInputText = nil

    self.interactionDelay = Scythe.INTERACTION_DELAY
    self.activatePressed = false

    return self
end

function Scythe:delete()
    g_effectManager:deleteEffects(self.effects)
    g_soundManager:deleteSample(self.sample)
    Scythe:superClass().delete(self)
end

function Scythe:postLoad(xmlFile)
    if not Scythe:superClass().postLoad(self, xmlFile) then
        return false
    end

    self.animations:loadFromXMLFile(xmlFile, "handTool", self.components, self.i3dMappings)

    local animation = {}
    animation.name = xmlFile:getValue("handTool.scythe.workingAnimation#name")
    animation.workingDirection = xmlFile:getValue("handTool.scythe.workingAnimation#workingDirection", 1)
    animation.releasingDirection = xmlFile:getValue("handTool.scythe.workingAnimation#releasingDirection", -animation.workingDirection)

    self.workingAnimation = animation

    self.effects = g_effectManager:loadEffect(xmlFile, "handTool.scythe.effects", self.components, self, self.i3dMappings)
    self.sample = g_soundManager:loadSampleFromXML(xmlFile, "handTool.scythe.sounds", "cut", self.baseDirectory, self.components, 0, AudioGroup.VEHICLE, self.i3dMappings, self)

    return true
end

function Scythe:update(dt, allowInput)
    Scythe:superClass().update(self, dt, allowInput)

    if self.isServer then
        local animation = self.workingAnimation
        if animation ~= nil then
            if self.animations:isPlaying(self.workingAnimation.name) then
                --local realArea, area, sprayFactor, plowFactor, limeFactor, weedFactor, stubbleFactor, rollerFactor, beeYieldBonusPerc, growthState, _, terrainDetailPixelsSum = FSDensityMapUtil.cutFruitArea(fruitTypeIndex, xs, zs, xw, zw, xh, zh, true, spec.allowsForageGrowthState, chopperValue)
                --
                --if realArea > 0 then
                --end
            end
        end
    end

    self.animations:update(dt)

    if self.isActive and allowInput then
        local interactionDelayReached = self.interactionDelay <= 0
        self:setIsCutting(self.activatePressed and interactionDelayReached)

        if interactionDelayReached then
            self.interactionDelay = Scythe.INTERACTION_DELAY
        end
    end

    self.interactionDelay = self.interactionDelay - dt
    self.activatePressed = false
end

---@return void
function Scythe:onActivate(allowInput)
    Scythe:superClass().onActivate(self, allowInput)
    self.originalPlayerInputText = nil
    self:setPlayerInputText(self.playerInputText)
end

---@return void
function Scythe:onDeactivate(allowInput)
    Scythe:superClass().onDeactivate(self, allowInput)

    if self.playerInputText ~= nil then
        self:setPlayerInputText(self.originalPlayerInputText)
    end
end

function Scythe:resetPlayerInputText()
    if self.playerInputText ~= nil then
        self:setPlayerInputText(self.playerInputText)
    else
        self:setPlayerInputText(self.originalPlayerInputText)
    end
end

---Set the player input text for the handtool
function Scythe:setPlayerInputText(text)
    local info = self.player.inputInformation.registrationList[InputAction.ACTIVATE_HANDTOOL]
    if self.originalPlayerInputText == nil then
        self.originalPlayerInputText = info.text
    end

    if text ~= nil then
        g_inputBinding:setActionEventText(info.eventId, text)
    end
end

function Scythe:setIsCutting(isCutting, force, noEventSend)
    --.sendEvent(self.player, isCutting, noEventSend)

    if self.isCutting ~= isCutting then
        if isCutting then
            g_effectManager:startEffects(self.effects)
            g_soundManager:playSample(self.sample)

            local animation = self.workingAnimation
            if animation ~= nil then
                self.animations:playAnimation(animation.name, animation.workingDirection)
            end
        else
            if force then
                g_effectManager:resetEffects(self.effects)
            else
                g_effectManager:stopEffects(self.effects)
            end

            g_soundManager:stopSample(self.sample)

            local animation = self.workingAnimation
            if animation ~= nil then
                self.animations:playAnimation(animation.name, animation.releasingDirection)
            end
        end

        self.isCutting = isCutting
    end
end
